
#    MIDAS Httpd Server to provide Data Acquisition Control Services

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

global MIDAS_DataAcquisitionControlServer_Implementation
set MIDAS_DataAcquisitionControlServer_Implementation 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0


global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
set MIDAS_DataAcquisitionCode 2

#    state returns

#    code (int)  state (string)  reason  (string)

#       0           stopped
#       1           going
#       2           reset    "please setup"
#       3           setting up
#       4           changing to going
#       5           changing to stopped
#       6           error           ******
#       7           changing to reset
#       8           undefined       ******


set MIDAS_DataAcquisitionState(0) "stopped"
set MIDAS_DataAcquisitionState(1) "going"
set MIDAS_DataAcquisitionState(2) "reset"
set MIDAS_DataAcquisitionState(3) "setting up"
set MIDAS_DataAcquisitionState(4) "changing to going"
set MIDAS_DataAcquisitionState(5) "changing to stopped"
set MIDAS_DataAcquisitionState(6) "error"
set MIDAS_DataAcquisitionState(7) "changing to reset"
set MIDAS_DataAcquisitionState(8) "undefined"


set MIDAS_DataAcquisitionReason(0) ""
set MIDAS_DataAcquisitionReason(1) ""
set MIDAS_DataAcquisitionReason(2) "please setup"
set MIDAS_DataAcquisitionReason(3) "please wait"
set MIDAS_DataAcquisitionReason(4) "please wait"
set MIDAS_DataAcquisitionReason(5) "please wait"
set MIDAS_DataAcquisitionReason(6) "try reset"
set MIDAS_DataAcquisitionReason(7) "please wait"
set MIDAS_DataAcquisitionReason(8) "try reset"


# Use namespaces to isolate our methods
namespace eval urn:DataAcquisitionControlServer {

    proc ServerVerbose {opt} {
        do_ServerVerbose $opt
        return [rpcvar::rpcvar int 0]
    }

    proc ServerImplementation {opt} {
        do_ServerImplementation $opt
        return [rpcvar::rpcvar int 0]
    }

    proc Get-LastError {args} {
        set vl [do_Get-LastError]
        return [list [list ResponseCode int [lindex $vl 0]] [list Code int [lindex $vl 1]]  [list Reason string [lindex $vl 2]]]
    }

    proc Get-Information {args} {
        set vl [do_Get-Information]
        return [list [list ResponseCode int [lindex $vl 0]] [list Reason string [lindex $vl 1]]]
    }

    proc Get-State {args} {
        set vl [do_Get-State]
        return [list [list ResponseCode int [lindex $vl 0]] [list Code int [lindex $vl 1]]  [list State string [lindex $vl 2]] [list Reason string [lindex $vl 3]]]
    }

    proc Do-SetUp {args} {
        set vl [do_Do-SetUp]
        return [rpcvar::rpcvar int $vl]
    }

    proc Do-Reset {args} {
        set vl [do_Do-Reset]
        return [rpcvar::rpcvar int $vl]
    }

    proc Do-Stop {args} {
        set vl [do_Do-Stop]
        return [rpcvar::rpcvar int $vl]
    }

    proc Do-Go {args} {
        set vl [do_Do-Go]
        return [rpcvar::rpcvar int $vl]
    }

    proc Get-Counters {args} {
        set vl [do_Get-Counters]
        return [list [list ResponseCode int 0] [list Counters string $vl]]
    }

    proc Get-Rates {args} {
        set vl [do_Get-Rates]
        return [list [list ResponseCode int 0] [list Rates string $vl]]
    }

    proc do_ServerVerbose {option} {
        global MIDAS_DataAcquisitionControlServer_Verbose
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "called method ServerVerbose with parameter $option"}
        set MIDAS_DataAcquisitionControlServer_Verbose $option
    }

    proc do_ServerImplementation {option} {
        global MIDAS_DataAcquisitionControlServer_Verbose  MIDAS_DataAcquisitionControlServer_Implementation
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "called method ServerImplementation with parameter $option"}
        set MIDAS_DataAcquisitionControlServer_Implementation $option
    }

    proc do_Get-State {} {
       global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set z [catch {inform sys Get_State} m]
       set env(EGsilent) 0

       if {$z == 0} then {
           return [list 0 [lindex $m 0] [lindex $m 1] [lindex $m 2]]
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return [list 1 8 "" ""]
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Get-LastError {} {
       global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set m 0x0000
       set i "No error information is available at present"
       set z 0
       set env(EGsilent) 0

       if {$z == 0} then {
          return [list 0 $m $i]
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return [list 1 0 ""]
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Get-Information {} {
       global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set m "No further informaton available at present"
       set z 0
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return [list 1 ""]
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Do-SetUp {} {
       global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set z [catch {inform sys do_Setup} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return $m
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Do-Reset {} {
       global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set z [catch {inform sys do_Reset} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return $m
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Do-Stop {} {
       global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set z [catch {inform sys do_Stop} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return $m
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Do-Go {} {
        global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set z [catch {inform sys do_Go} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return $m
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Get-Counters {} {
        global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set z [catch {inform stats Get_Counters} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return "0 0 0 0"
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }

    proc do_Get-Rates {} {
       global env MIDAS_DataAcquisitionControlServer_Implementation

       set env(EGsilent) 1
       set z [catch {inform stats Get_Rates} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           if {$MIDAS_DataAcquisitionControlServer_Implementation == 0} {
               return "0 0 0 0"
           } else {
               return -code error -errorcode 0x8005 -errorinfo "received error response: rc=$z\n  $m"
           }
       }
    }



    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export ServerImplementation
     SOAP::export Get-State
     SOAP::export Do-SetUp
     SOAP::export Do-Reset
     SOAP::export Do-Stop
     SOAP::export Do-Go
     SOAP::export Get-Counters
     SOAP::export Get-Rates
     SOAP::export Get-LastError
     SOAP::export Get-Information
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /DataAcquisitionControlServer -namespace urn:DataAcquisitionControlServer -uri urn:DataAcquisitionControlServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DataAcquisitionControlServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}
